Module: Zaius::Util

Defined in:
lib/zaius/util.rb

Constant Summary collapse

OPTS_USER_SPECIFIED =

Options that a user is allowed to specify.

Set[
  :api_key,
].freeze
OPTS_COPYABLE =

Options that should be copyable from one ZaiusObject to another including options that may be internal.

(
  OPTS_USER_SPECIFIED + Set[:api_base]
).freeze
OPTS_PERSISTABLE =

Options that should be persisted between API requests. This includes client, which is an object containing an HTTP client to reuse.

(
  OPTS_USER_SPECIFIED + Set[:client]
).freeze

Class Method Summary collapse

Class Method Details

.check_api_key!(key) ⇒ Object

Raises:

  • (TypeError)


193
194
195
196
# File 'lib/zaius/util.rb', line 193

def self.check_api_key!(key)
  raise TypeError, "api_key must be a string" unless key.is_a?(String)
  key
end

.check_string_argument!(key) ⇒ Object

Raises:

  • (TypeError)


188
189
190
191
# File 'lib/zaius/util.rb', line 188

def self.check_string_argument!(key)
  raise TypeError, "argument must be a string" unless key.is_a?(String)
  key
end

.convert_to_zaius_object(data, opts = {}) ⇒ Object

Converts a hash of fields or an array of hashes into a ZaiusObject or array of ZaiusObjects. These new objects will be created as a concrete type as dictated by their object field (e.g. an object value of charge would create an instance of Charge), but if object is not present or of an unknown type, the newly created instance will fall back to being a ZaiusObject.

Attributes

  • data - Hash of fields and values to be converted into a ZaiusObject.

  • opts - Options for ZaiusObject like an API key that will be reused on subsequent API calls.



58
59
60
61
62
63
64
65
66
67
68
# File 'lib/zaius/util.rb', line 58

def self.convert_to_zaius_object(data, opts = {})
  case data
  when Array
    data.map { |i| convert_to_zaius_object(i, opts) }
  when Hash
    # Try converting to a known object class.  If none available, fall back to generic ZaiusObject
    object_classes.fetch(data[:object], ZaiusObject).construct_from(data, opts)
  else
    data
  end
end

.encode_parameters(params) ⇒ Object

Encodes a hash of parameters in a way that’s suitable for use as query parameters in a URI or as form parameters in a request body. This mainly involves escaping special characters from parameter keys and values (e.g. ‘&`).



115
116
117
118
# File 'lib/zaius/util.rb', line 115

def self.encode_parameters(params)
  Util.flatten_params(params)
      .map { |k, v| "#{url_encode(k)}=#{url_encode(v)}" }.join("&")
end

.flatten_params(params, parent_key = nil) ⇒ Object



131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
# File 'lib/zaius/util.rb', line 131

def self.flatten_params(params, parent_key = nil)
  result = []

  # do not sort the final output because arrays (and arrays of hashes
  # especially) can be order sensitive, but do sort incoming parameters
  params.each do |key, value|
    calculated_key = parent_key ? "#{parent_key}[#{key}]" : key.to_s
    if value.is_a?(Hash)
      result += flatten_params(value, calculated_key)
    elsif value.is_a?(Array)
      result += flatten_params_array(value, calculated_key)
    else
      result << [calculated_key, value]
    end
  end

  result
end

.flatten_params_array(value, calculated_key) ⇒ Object



150
151
152
153
154
155
156
157
158
159
160
161
162
# File 'lib/zaius/util.rb', line 150

def self.flatten_params_array(value, calculated_key)
  result = []
  value.each_with_index do |elem, i|
    if elem.is_a?(Hash)
      result += flatten_params(elem, "#{calculated_key}[#{i}]")
    elsif elem.is_a?(Array)
      result += flatten_params_array(elem, calculated_key)
    else
      result << ["#{calculated_key}[#{i}]", elem]
    end
  end
  result
end

.log_debug(message, data = {}) ⇒ Object



77
78
79
80
81
82
# File 'lib/zaius/util.rb', line 77

def self.log_debug(message, data = {})
  if !Zaius.logger.nil? ||
     !Zaius.log_level.nil? && Zaius.log_level <= Zaius::LEVEL_DEBUG
    log_internal(message, data, color: :cyan, out: $stderr)
  end
end

.log_error(message, data = {}) ⇒ Object



84
85
86
87
88
89
# File 'lib/zaius/util.rb', line 84

def self.log_error(message, data = {})
  if !Zaius.logger.nil? ||
     !Zaius.log_level.nil? && Zaius.log_level <= Zaius::LEVEL_ERROR
    log_internal(message, data, color: :cyan, out: $stderr)
  end
end

.log_info(message, data = {}) ⇒ Object



70
71
72
73
74
75
# File 'lib/zaius/util.rb', line 70

def self.log_info(message, data = {})
  if !Zaius.logger.nil? ||
     !Zaius.log_level.nil? && Zaius.log_level <= Zaius::LEVEL_INFO
    log_internal(message, data, color: :cyan, out: $stderr)
  end
end

.normalize_headers(headers) ⇒ Object

Normalizes header keys so that they’re all lower case and each hyphen-delimited section starts with a single capitalized letter. For example, request-id becomes Request-Id. This is useful for extracting certain key values when the user could have set them with a variety of diffent naming schemes.



203
204
205
206
207
208
209
210
# File 'lib/zaius/util.rb', line 203

def self.normalize_headers(headers)
  headers.each_with_object({}) do |(k, v), new_headers|
    k = k.to_s.tr("_", "-") if k.is_a?(Symbol)
    k = k.split("-").reject(&:empty?).map(&:capitalize).join("-")

    new_headers[k] = v
  end
end

.normalize_id(id) ⇒ Object



164
165
166
167
168
169
170
171
172
# File 'lib/zaius/util.rb', line 164

def self.normalize_id(id)
  if id.is_a?(Hash) # overloaded id
    params_hash = id.dup
    id = params_hash.delete(:id)
  else
    params_hash = {}
  end
  [id, params_hash]
end

.normalize_opts(opts) ⇒ Object

The secondary opts argument can either be a string or hash Turn this value into an api_key and a set of headers



176
177
178
179
180
181
182
183
184
185
186
# File 'lib/zaius/util.rb', line 176

def self.normalize_opts(opts)
  case opts
  when String
    { api_key: opts }
  when Hash
    check_api_key!(opts.fetch(:api_key)) if opts.key?(:api_key)
    opts.clone
  else
    raise TypeError, "normalize_opts expects a string or a hash"
  end
end

.object_classesObject



39
40
41
42
43
44
# File 'lib/zaius/util.rb', line 39

def self.object_classes
  @object_classes ||= {
    ListObject::OBJECT_NAME => ListObject,
    Subscription::OBJECT_NAME => Subscription,
  }
end

.objects_to_ids(h) ⇒ Object



24
25
26
27
28
29
30
31
32
33
34
35
36
37
# File 'lib/zaius/util.rb', line 24

def self.objects_to_ids(h)
  case h
  when APIResource
    h.id
  when Hash
    res = {}
    h.each { |k, v| res[k] = objects_to_ids(v) unless v.nil? }
    res
  when Array
    h.map { |v| objects_to_ids(v) }
  else
    h
  end
end

.secure_compare(a, b) ⇒ Object

Constant time string comparison to prevent timing attacks Code borrowed from ActiveSupport



214
215
216
217
218
219
220
221
222
# File 'lib/zaius/util.rb', line 214

def self.secure_compare(a, b)
  return false unless a.bytesize == b.bytesize

  l = a.unpack "C#{a.bytesize}"

  res = 0
  b.each_byte { |byte| res |= byte ^ l.shift }
  res.zero?
end

.symbolize_names(object) ⇒ Object



91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
# File 'lib/zaius/util.rb', line 91

def self.symbolize_names(object)
  case object
  when Hash
    new_hash = {}
    object.each do |key, value|
      key = (begin
               key.to_sym
             rescue StandardError
               key
             end) || key
      new_hash[key] = symbolize_names(value)
    end
    new_hash
  when Array
    object.map { |value| symbolize_names(value) }
  else
    object
  end
end

.url_encode(key) ⇒ Object

Encodes a string in a way that makes it suitable for use in a set of query parameters in a URI or in a set of form parameters in a request body.



123
124
125
126
127
128
129
# File 'lib/zaius/util.rb', line 123

def self.url_encode(key)
  CGI.escape(key.to_s).
    # Don't use strict form encoding by changing the square bracket control
    # characters back to their literals. This is fine by the server, and
    # makes these parameter strings easier to read.
    gsub("%5B", "[").gsub("%5D", "]")
end